:root {
    --bokeh-launch-button-height: 28px;
  }

  /**
   * Rules related to the overall sidebar panel.
   */
  .bokeh-BokehDashboardLauncher {
    background: var(--jp-layout-color1);
    color: var(--jp-ui-font-color1);
    font-size: var(--jp-ui-font-size1);
    overflow: auto;
  }

  .bokeh-BokehDashboardLauncher-header {
    font-size: var(--jp-ui-font-size0);
    padding: 8px 12px;
    font-weight: 600;
    border-bottom: var(--jp-border-width) solid var(--jp-border-color2);
    letter-spacing: 1px;
    margin: 0px;
    text-transform: uppercase;
  }

  /**
   * Rules related to the dashboard launcher.
   */
  .nvdashboardListing-list {
    display: flex;
    flex-direction: column;
    justify-content: center;
    align-items: center;
    margin: 8px;
    padding: 0;
    list-style-type: none;
  }

  .nvdashboardListing-item {
    margin: 4px 0px 4px 0px;
    width: 100%;
  }

  .nvdashboardListing-item button {
    font-size: var(--jp-ui-font-size1);
    line-height: 24px;
    padding: 0px 8px;
    width: 100%;
  }

  .nvdashboardListing-item button.jp-mod-styled.jp-mod-accept {
    height: var(--bokeh-launch-button-height);
  }

  /**
   * Rules for the dashboard panels.
   */
  .bokeh-BokehDashboard-inactive {
    position: absolute;
    top: 0;
    left: 0;
    width: 100%;
    height: 100%;
    display: flex;
    align-items: center;
    justify-content: center;
    color: var(--jp-ui-font-color3);
    font-size: var(--jp-ui-font-size3);
    background-color: var(--jp-layout-color0);
    z-index: 10;
  }

  .bokeh-BokehDashboard-inactive:before {
    content: '';
    position: absolute;
    top: 0;
    left: 0;
    width: 100%;
    height: 100%;
    background-image: url(./expansion-card-variant-light.svg);
    background-repeat: no-repeat;
    background-position: center;
    opacity: 0.1;
  }

  /**
   * Icons
   */
  [data-jp-theme-light='true'] .jp-GPU-icon {
    background-image: url(./expansion-card-variant-light.svg);
  }
  [data-jp-theme-light="false"] .jp-GPU-icon {
    background-image: url(./expansion-card-variant-dark.svg);
  }
